
/* Saving data to file or sendout to netowrk with SUDP protocol.
 * Coded by $Author: sekido $
 * Ver. $Rev$  Date: $Date: 2011/08/18 07:47:12 $
 */


#ifndef LINUX
#define LINUX
#endif
#ifndef _LARGEFILE_SOURCE 
#define _LARGEFILE_SOURCE
#endif 
#define _LARGEFILE64_SOURCE

#include <cstdio>
#include <cstdlib>
#include <string>
using namespace std;

extern "C"{
  //#include <getopt.h>
#include <ctype.h>
}

#include "time_lib.hpp"

#include "InVSI_mdl.hpp"
#include "VLBI_Proto.hpp"
#include "VDIF_Proto.hpp"
#include "Raw_Proto.hpp"
#include "VLBI_Frame.hpp"
#include "VDIF_Frame.hpp"
#include "Dumper.hpp"
#include "Logsystem.hpp"
#include "StaticVals.hpp"
#include "conf_utl.hpp"

#define DEBUG(x) 
/** General Configulation data */
Config_set vdif_conf;

char *me;  // My name

// int Str2Tint(char *a){
//   int scale=1;
  
//   if (a[strlen(a)-1]=='h') {
//     scale=60*60; /* Hours -> seconds  */
//   } else if (a[strlen(a)-1]=='m') {
//     scale=60; /* Minutes -> seconds  */
//   }
//   int length=(uint32_t) ceil(atof(a)*scale);
//   return length;
// }

void usage(){
  printf("Usage: %s [options]\n",me);
  printf("  -s <Byte>        :  UDP block size\n");
  //  printf("  -f <vdif/m5b/k5> :  Data Frame type. Default(vdif)\n");
  printf("  -t <xs|xm|xh>    :  time length to sent\n");
  printf("  -p <PORT>        :  Port to use\n");
  printf("  -q <SUDP/VDIF>   :  Appl. Trans. Protocol. (Default SUDP)\n");
  printf("  -Q <udp/tcp/Raw>     :  Trans. Protocol.\n");
  printf("  -a <HOSTNAME>    :  Remote host name.\n");
  printf("     example: host1/host2. \n");
  printf("  -m <'xxMsps/xbit/xxch'>: Data mode\n");
  printf("        Msps -> [04Msps|08Msps|16Msps|32Msps|64Msps]\n");
  printf("        bit  -> 2bit only                       \n");

  printf("        ch   -> [16ch|08ch|04ch]                \n");
  printf("  -o <out file name>: Writing data to a file. \n");
  printf("     example: -o file1:file2. \n");
  printf("  -g <igrp>        : Group number 0-32/nch, default(0).\n");
  printf("      example: -g 0:1:2:3 , -g 0:1:2.\n");
  //  printf("  -v <np>          :  Verbose level (default=0).\n");
  printf("  -h                    This list\n");
  exit(1);
}

int main (int argc, char * const argv[]) {
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  
  /// Device Info Setup 
  string k5vsi_mode = "16Msps/2bit/08ch";

  vector <string> hostname;
  vector <string> fname;
  int port=DEFAULT_PORT;
  int mtu  = DEFAULT_MTU;
  if(argc < 2) usage();
  int tspan = 11;
  vector <int> igrp(1);
  int proto[2] = {TransInfo::SUDP_Proto, TransInfo::UDP_IO};
  
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hs:t:p:a:m:o:g:q:Q:")) { 
    case 'h':   usage();     break;
    case 'p':   port=atoi(optarg); break;
    case 'a':   hostname = StrSep(optarg,"/"); break;
    case 't':   tspan = Str2Tint(optarg); break;
    case 'g':   igrp = IntSep(optarg,":"); break;
    case 'm':   
      k5vsi_mode = optarg;
      fprintf(stderr,"Mode=%s\n",optarg); 
      break;
    case 'o':    
      fname = StrSep(optarg,":"); 	
      proto[0] = VLBI_Proto::RAW_Proto;
      proto[1] = VLBI_Proto::FILE_IO;
      break;
    case 'q':    
      if(strncmp(optarg,"VDIF",4)==0){
	proto[0] = TransInfo::VDIF_Proto;
	proto[1] = TransInfo::TCP_IO;
      }  
      break;
    case 'Q': {
      const int slen = strlen(optarg);
      char buf[slen+10];
      for(int i=0; i<slen; i++) buf[i] = toupper(optarg[i]); buf[slen]='\0';
      if(strncmp(optarg,"TCP",3)==0) proto[1] = TransInfo::TCP_IO;
      if(strncmp(optarg,"RAW",3)==0) proto[1] = TransInfo::FILE_IO;
      break;
    }
    case 's':  mtu = atoi(optarg); break;
      //    case 'v':   Logsystem::debug_logger()->Set_level(atoi(optarg)) ;    break; 
    case '?':  usage(); break;
    }
  }
  
  if(hostname.size() ==0 && fname.size() ==0){
    cerr << "No Output (file/net) Specified.\n";
    usage();
  }


  uint ngrp = igrp.size();
  K5_VSI k5vsi[ngrp];  // K5VSI device module 

  if(proto[1] == VLBI_Proto::FILE_IO && ngrp != fname.size()){
    fprintf(stderr,
	    "Error Number of group(%u) must be the same with #output files(%d)\n",
	    ngrp, (unsigned int)fname.size());
    return -1;
  }
  
  if( proto[1] != VLBI_Proto::FILE_IO && ngrp != hostname.size()){
    fprintf(stderr,
	    "Error Number of group(%u) must be the same with #hostnames(%d)\n",
	    ngrp, (unsigned int)hostname.size());
    return -1;
  }

  /// Setup protocol Output interface   
  // VLBI_Proto *tx_proto;
  //  VLBI_Proto vlbi_prot[4](port); // Using VDIF Header and protocol 
  VLBI_Proto *vlbi_pro[ngrp]; // Using VDIF Header and protocol 
//   if(proto[0] == TransInfo::VDIF_Proto)
//     for(uint i=0; i<ngrp; i++) vlbi_pro[i] = new VDIF_Proto(mtu);
  switch( proto[0]){
  case VLBI_Proto::RAW_Proto:
    for(uint i=0; i<ngrp; i++) vlbi_pro[i] = new RAW_Proto;
    break;
  case VLBI_Proto::SUDP_Proto:
  default:
    for(uint i=0; i<ngrp; i++) vlbi_pro[i] = new SUDPProto(mtu);
    break;
  }
  
  for(uint i=0; i < fname.size() && i < igrp.size(); i++){
    vlbi_pro[i]->Set_Host(fname[i].c_str());
    vlbi_pro[i]->Set_DataPort(port);
  }

  /// Setup data format/frame 
  VDIF_Frame vdif[ngrp];
  for(uint i=0; i<ngrp; i++){
    k5vsi[i].Set_Mode(k5vsi_mode.c_str());
    vdif[i].Set_Sparam(k5vsi[i].Nch(), k5vsi[i].Nbit(),k5vsi[i].Srate());
    vdif[i].Set_StnID("kb");
    vdif[i].Set_FrameDiv(10);
  }
			  
  Dumper dumper[ngrp];

  for(int i=0; i< (int)ngrp; i++){
    dumper[i].Set_Span(tspan);    
    //    k5vsi[i].Set_iGrp(igrp[i]);
    vlbi_pro[i]->Set_VlbiFrame(&vdif[i]);
    if( ! vlbi_pro[i]->Tx_Ready() ){
      cerr << "Error: VDIF-Protocol[" << i 
	   << "] is not ready\n"; continue; };

    if(dumper[i].Set_IO(vlbi_pro[i], &k5vsi[i]) <0){
      cerr << "Error Dumper::Set_IO()\n";
      return -1;
    }

    if(dumper[i].Ready()){
      fprintf(stderr, " Dump[%d] Starting.\n",i);
      dumper[i].start();
    }else{
      fprintf(stderr, " Dump[%d] Did not Started due to Ready() error.\n",i);
    }
  }
  
  
  for(uint j=0; j < ngrp; j++){
    if(dumper[j].join() !=0){
      cerr << "dumper[" << j << "] returned with error." << endl;
    }else
      fprintf(stderr, "dumper[%d] returned with success.\n", j);
  }

  return(0);
}

