
#include "InFile_mdl.hpp"
#include "Logsystem.hpp"
/** Constructor */

InFile_Model::InFile_Model():
  tds_file(0)
{
  SetID("InFile_Model");
}


void* InFile_Model::thread_func(){
  cerr << "Input Dev opend\n";
  
  switch(vlbi_frame->Type()){
  case VLBI_Frame::K5VSSP_frm:
    ReadingData_vssp();
    break;
  case VLBI_Frame::MK5B_frm:
    ReadingData_mk5b();
    break;
  case VLBI_Frame::K5VSI_frm:
    ReadingData_vsi();
    break;
  case VLBI_Frame::VDIF_frm:
    ReadingData_vdif();
    break;
  case VLBI_Frame::Undef_frm:
    cerr << "Error in InFile_Model::thread_func(): VLBI_frame type is Undef_frm.\n";
    break;
  }

  Stop(); // Including CloseDev();

//  Thread::state = Thread::ENDING;
  cerr << " Leaving InFile_mdl-thread..." << endl;

  //  Ready(); // For next calling 

  return 0;
}

int InFile_Model::ReadingData_vssp(){
  Logsystem::info_logger()->Tstamp() << "DataReading start\n";
  int tstep = 1000;
  if( tds_file == 0 ){ cerr << " tds_file is not ready, Ready() first! \n"; return -1;}
  tds_file->Set_tspan(tstep);  // 1 sec for buffer
  //  int ihdr =1;
  
  int n_loop = N_LOOP_LIMIT + 10;
  time_t t1(0);
  int ipage(0), next_page(0);
  int npage = dpage.Npage();
  //  uint32_t pagesize = dpage.Pagesize();
  tds_file->Set_NoInterpret();
  int ret(1);
  cerr << "InitialPageState :";  dpage.PrintPageState();

  int   nch = tds_file->Nch();
  int   nbit = tds_file->Sbit();
  int32_t sfrq = tds_file->Sfrq();
  int32_t Byte_sec = vlbi_frame->Set_Sparam(nch, nbit, sfrq);
  dpage.PageResize(Byte_sec);
  DAS_Info das = vlbi_frame->DAS_Inf();
  //  ret = tds_file->Read(tstep);
 
  //  while( n_loop > 0 && ret >= 0 ){
  while( n_loop > 0 && inputState != InDev_Model::STOP){
    //    cerr << "Reading data...";
    //    cerr << "ret=" << ret << ", page reading-end.\n";
    if( (t1 = tds_file->Current_hd().Epoch()) ==0) continue;
      
    ipage = t1 % npage;
    next_page = ( dpage.IpageWriting() +1 ) % npage;
    int nloop = N_LOOP_LIMIT + 10 ;
    if( next_page != ipage){
      while(dpage.IpageWriting() != ipage && ret >= 0 && nloop>0){ 
	ret = dpage.WStepPage(DataPage::EMPTY); // Put Empty status and Step to next page.
	if( ret < 0 ){
 	  nloop--;
	  Logsystem::debug_logger()->Tstamp() 
	    << "Waiting for Writing..Page[" << ipage << "](" 
	    << Epoch2Str(dpage.Epoch(ipage)) << ")";
	  Logsystem::debug_logger()->Flush();

	  dpage.PrintPageState();
	  //	  mutex_page.cond_timewait_2(1000); // 1sec
	  usleep(1000000);
	}
      }
    }else{
      while(nloop > 0){
	ret = dpage.WStepPage(DataPage::RD_OK); // Step to next page 
	if( ret < 0 ){ 	 
	  nloop--; 
	  Logsystem::debug_logger()->Tstamp() 
	    << "Waiting for Writing..Page[" << ipage << "](" 
	    << Epoch2Str(dpage.Epoch(ipage)) << ")"
	    << dpage.StrPageState()  << "\n";
	  Logsystem::debug_logger()->Flush();
	  
	  //	  mutex_page.cond_timewait_2(1000); // 1sec 
	  usleep(1000000);
	}else{
	  //	  mutex_page.cond_signal();
	  nloop =0;
	}
      }
    }
    if( ret < 0 ){
      Logsystem::debug_logger()->Tstamp() << "Page update timeout...";
      Logsystem::debug_logger()->Flush();
	  
      cerr << "Page update timeout...";
      break; //      continue;
    }

    if( Logsystem::Test_Logsys(Logsystem::INFO) ){
      struct tm tm1;
      char buf[256];
      gmtime_r(&t1,&tm1);
      strftime( buf, 256, "%Y/%j-%H:%M:%S", &tm1);
      Logsystem::info_logger()->Tstamp() 
	<< "Read data (" << das.Mode()  << ")[" << buf << "] and writing to Page:"
	<< ipage  << "\n";
      Logsystem::info_logger()->Flush();
    }
    //      dpage.tarr[ipage] = t1;
    dpage.SetTimeWritingPage(t1);
    vector <uint8_t>& pbuf = dpage.WritingPagePtr();
    /** Copying raw data from inner buffer to a user buffer */
    if( tds_file->Get_rawdata( pbuf ) <= 0 ){
      cerr << "E InFile_Model::ReadingData_vssp(): Get_rawdata() failed\n";
      Logsystem::info_logger()->Tstamp()
	<< "E InFile_Model::ReadingData_vssp(): Get_rawdata() failed.\n";
      Logsystem::info_logger()->Flush();
      dpage.ChgPageState(ipage, DataPage::DATA_END);
      break;
    }
    n_loop = N_LOOP_LIMIT + 10;
      //  } 
    
  } // While loop

  if( tds_file->Eof() ){
    cerr << "InFile_Model::ReadingData_vssp(): EOF found\n";
  }else{
    cerr << "E InFile_Model::ReadingData_vssp(): tds_file->Read() failed."
	 << strerror(errno) << endl;
  }
  dpage.WStepPage(DataPage::DATA_END); 

  cerr << " Leaving from ReadingData_vssp()...";
  return 0;
}


int InFile_Model::ReadingData_mk5b(){
  cerr << "Not Implemented." << endl;
  return 0;
}

int InFile_Model::ReadingData_vdif(){
  cerr << "Not Implemented." << endl;
  return 0;
}

int InFile_Model::ReadingData_vsi(){
  cerr << "Not Implemented." << endl;
  return 0;
}


/** Destructor */
InFile_Model::~InFile_Model(){
  if(tds_file != 0) delete tds_file;
  tds_file =0;
}

int InFile_Model::InitPage(){
  if(vlbi_frame ==0 ){
    cerr << "Error in void InFile_Model::InitPage(): vlbi_frame is empty."
	 << endl;
    return -1;
  }
  DAS_Info dasinfo =  vlbi_frame->DAS_Inf();
  uint32_t pagesize = dasinfo.Speed()*4;
  dpage.PageResize(pagesize);
  dpage.InitPages();
  return 0;
}

const int InFile_Model::TestDev() {
  if(fopen(infile.c_str(), "r") == 0){ return -1;}
  
  if(vlbi_frame ==0){
    Logsystem::error_logger()->Tstamp() 
      << "E InFile_Model::TestDev(): VLBI_Frame instance is not set yet.\n";
    cerr << "E InFile_Model::TestDev(): VLBI_Frame instance is not set yet.\n";
    return -1;
  }
  
  int ret(0);
  VLBI_Frame::VLBI_Frame_t vf_type = vlbi_frame->Type();
  switch(vf_type){
  case  VLBI_Frame::K5VSSP_frm :{
    TDS_FILE tds;
    ret = tds.Initial_Check(infile.c_str());
    if(ret > 0){
      TDS_HD& hd = tds.Current_hd();
      DAS_Info das(hd.Sfrq(), hd.Nch(), hd.Bit());
      vlbi_frame->Set_DasInfo(das);
      ret = 1;
    }
    break;
  }
  case VLBI_Frame::K5VSI_frm :
  case VLBI_Frame::VDIF_frm :
  case VLBI_Frame::MK5B_frm :
  case VLBI_Frame::Undef_frm : 
    break;
  }

  return ret;  

}


const int InFile_Model::OpenDev(){

  VLBI_Frame::VLBI_Frame_t vf_type = vlbi_frame->Type();
  int ret(0);
  //  try{
  switch(vf_type){
  case VLBI_Frame::K5VSSP_frm : {
    if(tds_file != 0) delete tds_file;
    tds_file = new TDS_FILE;
    ret = tds_file->Open(infile.c_str());
    int tspan=1000;
    tds_file->Set_tspan(tspan, true); // 1 sec for buffer
    if(ret <0){
      cerr << "Error in InFile_Model::OpenDev():tds_file->Open() failed"
	   << endl;
    }
    //      if(ret<0) throw "tds_file->Open() failed";
    break;
  }
  case VLBI_Frame::K5VSI_frm :
    ret = open(infile.c_str(),O_RDONLY);
    if(ret < 0){
      cerr << "Error in InFile_Model::OpenDev():K5VSI open() failed"
	   << endl;
    }
    //      if(ret<0) throw "K5VSI open() failed";
    break;
  case VLBI_Frame::VDIF_frm :
    cerr << "Error in InFile_Model::OpenDev(): VDIF_frm is not supported yet\n";
    ret = -1;
    break;
  case VLBI_Frame::MK5B_frm :
    cerr << "Error in InFile_Model::OpenDev(): Mk5B_frm is not supported yet\n";
    ret = -1;
    break;
  default:
    cerr << "Not yet Implemented in InFile_Model::OpenDev().\n";
    ret = -1;
    break;
  }
  //  }


  //   catch(const char* str){
  //     cerr << "Error in InFile_Model::OpenDev(): " << str << endl;
  //     return ret;
  //   }

  if(ret > 0) dpage.InitPages();
  return ret;  
  
}

const int InFile_Model::CloseDev(){
  VLBI_Frame::VLBI_Frame_t vf_type = vlbi_frame->Type();
  
  switch(vf_type){
  case VLBI_Frame::K5VSSP_frm : 
    if(tds_file !=0) delete tds_file; 
    tds_file=0;    break;
  case VLBI_Frame::K5VSI_frm : 
    break;
  case VLBI_Frame::MK5B_frm : 
    break;
  case VLBI_Frame::VDIF_frm : 
    break;
  case VLBI_Frame::Undef_frm : 
    break;
  }
  
  //  mutex_page.unlock();
  dpage.MutexRelease();

  return 0;
}

// /** MappingData()
//  * Mapping the memory to data array
//  * @param pntr: reference of vector array 
//  * @param epoch: data Epoch 
//  *  - Epoch is used for paging. Epoch % Npage = ipage
//  */
// const InFile_Model::IndevReturns_t 
// InFile_Model::MappingData(vector <uint8_t>& pntr, const time_t epoch){
//   //  dpage.PrintPageState();
//   int ipage = epoch % dpage.npage;
//   if(dpage.status[ipage] == DataPage::DATA_END){
//     cerr << "InFile_Model::MappingData(): END Found.\n";
//     return DATA_END;
//   }
//   //  cerr << "Mapping " << ipage << ": ";    dpage.PrintPageState();
//   time_t t1 = dpage.Epoch(ipage);
//   int nloop = N_LOOP_LIMIT;
//   bool tcond = t1 < epoch;
//   bool pcond = dpage.status[ipage] == DataPage::EMPTY ||
//     dpage.status[ipage] == DataPage::R_PROTECTED;
  
//   while( (pcond || tcond)  && nloop > 0){
//     //   while( (dpage.status[ipage] == DataPage::WR_OK ||
//     // 	  dpage.status[ipage] == DataPage::RD_OK) &&
//     // 	 t1 != epoch && nloop > 0 ){ 
//     Logsystem::debug_logger()->Tstamp() 
//       << "Waiting for Reading..Page[" << ipage <<  "](" 
//       << Epoch2Str(epoch) << ")"
//       <<  dpage.StrPageState()  << endl ;
//     struct timeval now;
//     struct timespec timeout;
//     gettimeofday(&now,0);
//     timeout.tv_sec = now.tv_sec + 1;
//     timeout.tv_nsec = now.tv_usec * 1000;
//     mutex_page.cond_timewait(&timeout);
//     t1 = dpage.Epoch(ipage);
//     tcond = t1 < epoch;
//     pcond = dpage.status[ipage] != DataPage::EMPTY || 
//       dpage.status[ipage] != DataPage::R_PROTECTED;
//     nloop --;
//   }

//   if( nloop == 0 ){
//     char buf[2][256];
//     struct tm tm1, tm2;
//     gmtime_r(&epoch, &tm1);
//     gmtime_r(&t1, &tm2);
//     strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1);
//     strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
//     Logsystem::error_logger()->Tstamp() 
//       << "Error at InFile_Model::Mapping(): Failed to get expected time." 
//       << "Data(" << buf[0] << ") but (" << buf[1] << ")\n";
//     return GEN_ERROR;
//   }
  
//   if(t1 != epoch ){
//     char buf[2][256];
//     struct tm tm1, tm2;
//     gmtime_r(&epoch, &tm1);
//     gmtime_r(&t1, &tm2);
//     strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1);
//     strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
//     fprintf(stderr,"Warining in InFile_Model::MappingData(): Time of page(%s) != requested(%s)\n",
// 	    buf[1],buf[0]);
//     return GEN_ERROR;
//   }
//   pntr = dpage.dat[ipage];
//   return NORMAL;

// }


const uint8_t* InFile_Model::MappingData(time_t epoch){
  string func_id = "InFile_Model::MappingData():";

  int ipage = epoch % dpage.Npage();
  if( dpage.TestPage(ipage, DataPage::DATA_END) ){
    Logsystem::error_logger()->Tstamp()
      << "E " << func_id << " END Found.\n";
    cerr << "E " << func_id << " END Found.\n";
    return 0;
  }
  //  cerr << "Mapping " << ipage << ": ";    dpage.PrintPageState();
  time_t t1 = dpage.Epoch(ipage);
  int nloop = N_LOOP_LIMIT;
  bool tcond = t1 < epoch;
  bool pcond = dpage.TestPage(ipage, DataPage::EMPTY) ||
    dpage.TestPage(ipage, DataPage::R_PROTECTED);
  
  while( (pcond || tcond)  && nloop > 0){
    //   while( (dpage.status[ipage] == DataPage::WR_OK ||
    // 	  dpage.status[ipage] == DataPage::RD_OK) &&
    // 	 t1 != epoch && nloop > 0 ){ 
    Logsystem::debug_logger()->Tstamp() 
      << "I " << func_id << "Waiting for "<< Epoch2Str(epoch) 
      << " at Reading..Page[" << ipage <<  "] =("  << Epoch2Str(t1) 
      << ")" <<  dpage.StrPageState() << "\n" ;
    Logsystem::debug_logger()->Flush();
//     struct timeval now;
//     struct timespec timeout;
//     gettimeofday(&now,0);
//     timeout.tv_sec = now.tv_sec + 1;
//     timeout.tv_nsec = now.tv_usec * 1000;
//     mutex_page.cond_timewait(&timeout);
    usleep(1000000);
    t1 = dpage.Epoch(ipage);
    tcond = t1 < epoch;
//     pcond = dpage.PageState(ipage) != DataPage::EMPTY || 
//       dpage.PageState(ipage) != DataPage::R_PROTECTED;
    pcond = dpage.TestPage(ipage, DataPage::EMPTY) ||
      dpage.TestPage(ipage, DataPage::R_PROTECTED);
    nloop --;
  }

  if( nloop == 0 ){
    char buf[2][256];
    struct tm tm1, tm2;
    gmtime_r(&epoch, &tm1);
    gmtime_r(&t1, &tm2);
    strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1);
    strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
    Logsystem::error_logger()->Tstamp() 
      << "Error at InFile_Model::Mapping(): Failed to get expected time." 
      << "Data(" << buf[0] << ") but (" << buf[1] << ")\n";
    return 0;
  }
  
  if(t1 != epoch ){
    char buf[2][256];
    struct tm tm1, tm2;
    gmtime_r(&epoch, &tm1);
    gmtime_r(&t1, &tm2);
    strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1);
    strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
    fprintf(stderr,"Warining in InFile_Model::MappingData(): Time of page(%s) != requested(%s)\n",
	    buf[1],buf[0]);
    //    return GEN_ERROR;
    return 0;
  }
  //  pntr = dpage.dat[ipage];
  //  return &(dpage.dat[ipage][0]);
  return dpage.Data(ipage);
}


const InFile_Model::IndevReturns_t  
InFile_Model::UnMapping(const time_t data_epoch){
  int ipage = data_epoch % dpage.Npage();
  time_t t1 = dpage.Epoch(ipage);
  IndevReturns_t ret =NORMAL;
  if( t1 != data_epoch ){
    char buf[2][256];
    struct tm tm1, tm2;
    gmtime_r( &data_epoch, &tm1 );
    gmtime_r( &t1, &tm2 );
    strftime( buf[0],256, "%Y/%j-%H:%M:%S", &tm1 );
    strftime( buf[1],256, "%Y/%j-%H:%M:%S", &tm2 );
    fprintf(stderr, "Warining in InFile_Model::UnMapping: data epoch at page(%d) from %s to %s \n",
	    ipage, buf[0],buf[1]);
    ret = GEN_WARNING;
  }
  //  cerr << "UnMapping before" << ipage << ": ";    dpage.PrintPageState();
  dpage.ChgPageState(ipage, DataPage::WR_OK);
  //  cerr << ".....After: ";    dpage.PrintPageState();
  //  mutex_page.cond_signal();
  return ret;
}

int InFile_Model::Set_DasInfo(const DAS_Info& inf){
  Logsystem::error_logger()->Tstamp()
    << "Warning @InFile_Model::Set_DasInfo(): Faild to set das, "
    << "because DAS inf comes from file.\n";
  return -1;
}


int InFile_Model::Set_infile(const char * fname){
  infile = fname;
  int ret = TestDev();
  return ret;
}
