#ifndef __INMEM_MDL_HPP__
#define __INMEM_MDL_HPP__
/** Input Device (Memory) class
 *
 */


#include "Thread.h"
#include "InDev_mdl.hpp"
#include "DataPage.hpp"

extern "C"{
#include <unistd.h>
}

/** 
 * Input model for Memory reading. Succeeding from InDev_Model.
 */

class InMem_Model:public InDev_Model{
public:
  InMem_Model():time_span(10){SetID("InMem_Model");} ///< Constructor
  InMem_Model(const int tspan):time_span(tspan){;} ///< Constructor
    ~InMem_Model(){;} ///< Destructor 
  /// Mapping data page of the epoch to reference of vector array
    const IndevReturns_t MappingData(vector <uint8_t>& pntr, const time_t epoch=0);
  /// Mapping data page, pinter verison
  const uint8_t* MappingData(time_t epoch); 
  /// Release mapped page
  const IndevReturns_t  UnMapping(const time_t data_epoch);
  /**  page_length is page_size(1sec) in 4-byte unit  */
  uint32_t PageLength(){  return dpage.Pagesize()/4;}
  const int OpenDev(); ///< Open device/file to be ready 
  const int CloseDev(){ return 0;}  ///< Closing device
  const int TestDev(){return 0;} ///< Test device/file Permission  
  /// Returns Device type instring 
  string StrDevType(){string ret = "MEMORY"; return ret;}
  /// Returns Device type 
  InputModelType_t DevType(){return InDev_Model::MEM;}
  int InitPage(); ///< Initializing Page size with DAS_info
  /// Returns Epoch of the i page;  
  time_t Epoch(const int i)const{ return dpage.Epoch(i);}
  /// Return page number of the earliest epoch
  int FirstPage() {time_t t1 = dpage.EarliestEpoch();
    if( t1 == 0) return -1; else return (t1 % dpage.Npage());}
  /// Returns Page sate in string 
  string StrPageState(const int lvl=0){return dpage.StrPageState(lvl);}
  void PageEnd(); ///< Set DATA_ENbD state  to all pages
  void* thread_func();
  /** function to return number of pages.   */
  const int  Npage(){return dpage.Npage();}
private:
  int MemPageUpdate();
  DataPage dpage; 
  int time_span; // (sec)
  InMem_Model(InMem_Model& mdl){;}///< Copy Constructor, prohibited.
};



#endif
