

#ifndef  __TRANS_CTL_HPP__
#define  __TRANS_CTL_HPP__

#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <string>
#include <vector>
#include <boost/algorithm/string.hpp>
#include "Thread.h"
#include "DAS_info.hpp"
#include "Trans_info.hpp"
#include "Logsystem.hpp"
#include "Dumper.hpp"
#include "bitutl.hpp"
//#include "SubGrpInfo.hpp"

extern "C"{
#include <errno.h>
}

/** TCP/IP communication server module for Transmission Cotrol 
 * $Author: sekido $
 * $Revision: 1.3 $  $Date: 2011/09/09 04:12:33 $
 * $Id: Trans_ctl.hpp,v 1.3 2011/09/09 04:12:33 sekido Exp $
 */

class TransCtl: public Thread{
public:
  TransCtl(); ///<   Constructor 
  ~TransCtl(){Clean();} ///<  Destructor
  void Clean(); ///< Clean up for destructor
  /** Status of the protocol */
  enum pro_stat_t {IDLE, SERVING, STOP,TERM};
  /** Retuens protol state */
  pro_stat_t CtrlState()const {return ctrl_state;}
  string StrCtrlState()const;
  //  void Set_DasInfo(DAS_Info* ds){das_info = ds;} ///< Setup DAS_Info 
  /*!  Setup  Application Porotocl 
    \param *vp VLBI Protocol instance's address 
    \param ng  Number of Output streams. ng < vp[] array size;
   */
  int  Set_ApplProto(VLBI_Proto* vp[], const int ng=1); 
  /// Set Input device class pointer 
  void Set_InputDev(InDev_Model* inv);
  bool SetupCheck(); ///< Checking Setup
  //  const pro_stat_t State() const {return state;}
  int CtrlPort() const {return ctrl_port;} ///< Return Control Port number
  /// Set Control port number
  void Set_CtrlPort(const int p) {ctrl_port=p;}
  /// Return Dumper State
  string DumperState();
  /// Terminating 
  void Terminate();
private:
  TCPSocket* sock_;
  pro_stat_t ctrl_state;
  //  DAS_Info* das_info; ///< Data sampling information
  //  VLBI_Frame* vlbi_frame; ///< VLBI_frame 
  vector <VLBI_Proto*> out_protocol; ///< Netowrk Transport Information
  InDev_Model* indev;  ///< Input Device Module 
  int ngrp;    ///< Number of MAX groups for outputs
  int nout;    ///< Number of output groups 
  int nin;     ///< Number of input groups 
  int ctrl_port;   ///< Control Port 
  vector <Dumper*>     dumper;  ///< data Output Module
  //  vector <Dumper>  dumper;  ///< data Output Module
  //  vector <SubGrpInfo> out_subgrp; ///< Information of DAS SubGroups.
  //  vector <SubGrpInfo> in_subgrp; ///< Information of DAS SubGroups.
  /*!    Thread Function */
  void* thread_func(); ///<   Thread for server 
  /** Making Listen for control port */
  int Listen();///<  setup server socket to Listen 
  /**  Communication funciton with clients */
  int CommunicateWithClient(); ///< TCP Communication Server 
  /** Command perser 
   * @param *cmd Command char pointer
   * @param *sock Socket to return reply to the client.
   * @return 0 if success, othewise -1 
   */
  const int CmdParser(const char* cmd, Socket *sock);  ///< Command Parser
  // Prohibiting Copy Constructor 
  TransCtl(const TransCtl& tc){;}
};


inline string TransCtl::DumperState(){
  string buf;
  for(int i=0; i< (int)dumper.size(); i++){
    //  for(int i=0; i< (int)ndumper; i++){
    buf += dumper[i]->StrStatus() + " ";
  }
  return buf;
}

inline void TransCtl::Terminate(){
  
  for(int i=0; i< (int)dumper.size(); i++){
    dumper[i]->Stop();
  }
  ctrl_state = TERM;
  // ctrl_state = STOP;
}

inline string TransCtl::StrCtrlState()const{
  string ret;
  switch(ctrl_state){
  case IDLE: ret ="IDLE"; break;
  case SERVING: ret = "SERVING"; break;
  case STOP: ret = "STOP"; break;
  case TERM: ret = "TERM"; break;
  default: ret = "UNDEF"; break;
  }
  return ret;
}
#endif
