
#include "VLBI_Header.hpp"


//======= VSSP32 libraries ==== from here ======
void  K5VSSP_Hdr::Disp() {
  fprintf(stderr,"sync=%0X sync2=%x \n",sync_word, sync2);
  fprintf(stderr,"Epoch:%04d/%03d sec1=%0d sec2=%d \n",year,doy,sec1,sec);
  fprintf(stderr,"nch=%x sfreq=%x qbit=%x \n",nch_b, sfreq_b, qbit_b);
  fprintf(stderr,"ver=%d.%d size=%lu\n",ver_1, ver_2, sizeof(K5VSSP_Hdr));

  Print_Aux_Info();

  uint32_t *pt=(uint32_t*) this;
  for(int i=0; i<8; i++){ fprintf(stderr, "c(hex)= %0X ", *pt++); }
  putc('\n',stderr);
}

void K5VSSP_Hdr::Print_Aux_Info(FILE *fp){
  fprintf(fp, " AuxFmt= %d Fltr= %d MHz", 
	  aux_data.aux_fmt_No, aux_data.fltr_freq );

  switch(aux_data.aux_fmt_No ){
  case 2:{
    K5_Aux_2 *paux_2 = (K5_Aux_2*)&aux_data; 
    char pcname[80]; strncpy(pcname,paux_2->hostPCname,8); pcname[9]='\0';
    fprintf(fp," Host= %8s", pcname);
  } break;
  case 1:
    char pcname[80]; strncpy(pcname,aux_data.hostPCname,8);pcname[8]='\0';
    char stationname[80]; strncpy(stationname,aux_data.stname,8);stationname[8]='\0';
    char stid[80]; strncpy(stid,aux_data.station_id,2);stid[2]='\0';
    fprintf(fp," Host= %8s Stid: %2s Stname: %8s", pcname, stid, stationname );
    break;
  }

}

int32_t  K5VSSP_Header::Hd_find(const uint8_t*  buf, const uint32_t nbyte){
  int32_t remain = nbyte;
  int find_flag=0; 
  uint8_t *pb = (uint8_t* )buf;
  int32_t count=0;

  int32_t hdsize = Headsize();
  //  int32_t hdsize = aaa();
  while( remain >= hdsize
	 && find_flag==0){
    pt_head  = (K5VSSP_Hdr*)pb;
    if(dbg_disp_level>10) pt_head->Disp();
    if( pt_head->sync_word == 0xffffffff &&
	(pt_head->sync2 == 0x8b || pt_head->sync2 == 0x8c)){
      find_flag =1;
      memcpy(&head_buf[0], pb, Headsize());
      pt_head = (K5VSSP_Hdr*)&head_buf[0];
      return(count);
    }else{
      pb++; count++; remain--;
    }
  } // end while 
  pt_head=0;
  return(-1);
}

Time_dat K5VSSP_Header::Epoch(){
  Time_dat t1;
  if(pt_head == 0) {
    cerr << "Error in K5VSSP_Header::Epoch(): Header data is not ready\n";
    return t1;
  }

  if(Header_type() == K5VSSP32_Header_Type){
    double soy = (pt_head->Doy()-1)*86400. + pt_head->Sod();
    t1.Set_time(pt_head->Year(),soy);
  }else{  // Old VSSP format
    if(year==0){
      Time_dat t0;
      t0.Set_time_to_now();
      year = t0.Year();
      doy = (int)t0.Doy();
    }
    double soy = doy*86400. + pt_head->Sod();
    t1.Set_time(year,soy);
  }

  return t1;
}


void K5VSSP_Header::Print_hd(FILE *fp){
  if(pt_head ==0){
    cerr << "Error in K5VSSP_Header::Print_hd(): header data is not ready\n"; 
    return;
  }

  
  if(Header_type() == K5VSSP32_Header_Type){
    fprintf(fp, "- K5VSSP32 HD info:");
    fprintf(fp," Epoch= %s", Epoch().Print_time(20).c_str());
    fprintf(fp, " Nch= %d Sampl= %u Bit= %d", 
	    pt_head->Nch(), pt_head->Sfreq(), pt_head->Qbit() );
    fprintf(fp," Ver: %d.%d", pt_head->ver_1, pt_head->ver_2);
    pt_head->Print_Aux_Info(fp);

  }else{
    fprintf(fp, "- K5VSSP HD info:");
    fprintf(fp," Epoch= %s", Epoch().Print_time(20).c_str());
    //    fprintf(fp," Sod: %d", pt_head->Sod());
    fprintf(fp, " Nch= %d Sampl= %u Bit= %d", 
	    pt_head->Nch(), pt_head->Sfreq(), pt_head->Qbit() );
  }

  putc('\n',fp);

}


//======= VSSP32 libraries ==== to here ======

//======= Mk5B libraries ==== from here ======

int32_t  Mk5B_Header::Hd_find(const uint8_t*  buf, const uint32_t nbyte){
  uint32_t remain = nbyte;
  int find_flag=0; 
  uint8_t *pb = (uint8_t* )buf;

  uint32_t hdsize = Headsize();
  int32_t count=0;

  while( remain >= hdsize && find_flag==0){
    pt_head = (Mk5B_Hdr*)pb;
    if( pt_head->sync_word == 0xABADDEED ){
      find_flag =1;
      memcpy(&head_buf[0], pt_head, Headsize());
      return( count);
    }else{
      pb++; count++; remain--;
    }
  } // end while 
  pt_head=0;
  return(-1);
}

Time_dat Mk5B_Header::Epoch(){
  Time_dat t1;
  if(pt_head ==0){
    cerr << "Error in Mk5B_Header::Epoch(): pf_head is not ready\n";
    return t1;
  }
  //  double sod = Sod();
  Time_dat t0; t0.Set_time_to_now();
  double mjd_now = t0.Mjd();
  double mjd_tmp = (double)(pt_head->mjd2*10 + pt_head->mjd1)*10 + pt_head->mjd0;
  double mjd = (int)floor(mjd_now / 1000)*1000 + mjd_tmp;
  double sod = (((pt_head->s4*10+pt_head->s3)*10+pt_head->s2)*10+pt_head->s1) *10 +pt_head->s0;
  sod += (((pt_head->ms4*0.1 + pt_head->ms3)*0.1 + pt_head->ms2)*0.1 + pt_head->ms1)*0.1;
  mjd += sod/86400.;
  t1.Set_time(mjd);
  return t1;
}



void Mk5B_Header::Print_hd(FILE *fp){
  if(pt_head ==0){
    cerr << "Error in Mk5B_Header::Print_hd(): pt_head is not ready\n"; 
    return;
  }
  
  fprintf(fp, "- Mk5B HD info : ");
  //  fprintf(fp," Epoch-sec = %d/%d %d sec\n", year, doy, sod);
  fprintf(fp," Epoch = %s ", Epoch().Print_time(20).c_str());
  if(pt_head->tvg_flag) fprintf(fp,"TVG: ON  "); 
  else                  fprintf(fp,"TVG: OFF "); 
  fprintf(fp, " iframes=%d \n", pt_head->iFrame);
  //  fprintf(fp,"----------------------------\n");
}

//======= Mk5B libraries ==== to here ======

//======= VDIF libraries ==== from here ======


int32_t  VDIF_Header::Hd_find(const uint8_t* buf, const uint32_t nbyte){
  uint32_t remain = nbyte;
  int find_flag=0; 
  uint8_t *pb = (uint8_t* )buf;
  uint32_t hdsize = Headsize();
  int32_t count=0;

  while( remain >= hdsize && find_flag==0){
    pt_head = (VDIF_Hdr *)pb;
    if( pt_head->exud_nict.ext_dat_ver == 1
	&& pt_head->exud_nict.sync == VDIF_NICT_SYNC ){
      find_flag =1;
      memcpy(&head_buf[0], pt_head, Headsize());
      return(count);
    }else{
      pb++; count++; remain--;
    }
  } // end while 

  pt_head=0;
  return(-1);
}

Time_dat VDIF_Hdr::Ref_Epoch(){
  Time_dat t1;
  switch(Ref_Epoch_hy_counter){
  case 0:  t1.Set_time(2000,1,1,0,0,0);break;
  case 1:  t1.Set_time(2000,7,1,0,0,0);break;
  case 2:  t1.Set_time(2001,1,1,0,0,0);break;
  case 3:  t1.Set_time(2001,7,1,0,0,0);break;
  case 4:  t1.Set_time(2002,1,1,0,0,0);break;
  case 5:  t1.Set_time(2002,7,1,0,0,0);break;
  case 6:  t1.Set_time(2003,1,1,0,0,0);break;
  case 7:  t1.Set_time(2003,7,1,0,0,0);break;
  case 8:  t1.Set_time(2004,1,1,0,0,0);break;
  case 9:  t1.Set_time(2004,7,1,0,0,0);break;
  case 10:  t1.Set_time(2005,1,1,0,0,0);break;
  case 11:  t1.Set_time(2005,7,1,0,0,0);break;
  case 12:  t1.Set_time(2006,1,1,0,0,0);break;
  case 13:  t1.Set_time(2006,7,1,0,0,0);break;
  case 14:  t1.Set_time(2007,1,1,0,0,0);break;
  case 15:  t1.Set_time(2007,7,1,0,0,0);break;
  case 16:  t1.Set_time(2008,1,1,0,0,0);break;
  case 17:  t1.Set_time(2008,7,1,0,0,0);break;
  case 18:  t1.Set_time(2009,1,1,0,0,0);break;
  case 19:  t1.Set_time(2009,7,1,0,0,0);break;
  case 20:  t1.Set_time(2011,1,1,0,0,0);break;
  case 21:  t1.Set_time(2011,7,1,0,0,0);break;
  case 22:  t1.Set_time(2012,1,1,0,0,0);break;
  case 23:  t1.Set_time(2012,7,1,0,0,0);break;
  case 24:  t1.Set_time(2013,1,1,0,0,0);break;
  case 25:  t1.Set_time(2013,7,1,0,0,0);break;
  case 26:  t1.Set_time(2014,1,1,0,0,0);break;
  case 27:  t1.Set_time(2014,7,1,0,0,0);break;
  case 28:  t1.Set_time(2015,1,1,0,0,0);break;
  case 29:  t1.Set_time(2015,7,1,0,0,0);break;
  }

  return t1;
}

Time_dat VDIF_Header::Epoch(){
  Time_dat t0;
  if(pt_head ==0){
    cerr << "Error in VDIF_Header::Epoch(): Header is not ready\n";
    return t0;
  }

  t0 = pt_head->Ref_Epoch();
  t0 += pt_head->Sec_from_Ref;
  return t0;
}


void VDIF_Header::Print_hd(FILE *fp){
  if(pt_head ==0){
    cerr << "Error in VDIF_Header::Print_hd(): Header is not ready\n";
    return ;
  }

  fprintf(fp, "- VDIF HD (Ver. %d.%d)info:", 
	  pt_head->Version_No, pt_head->exud_nict.ext_dat_ver);
  //  fprintf(fp," Epoch-sec = %d/%d %d sec\n", year, doy, sod);
  if(pt_head->Invalid_flag){  fprintf(fp," Invalid data:");}
  else{                       fprintf(fp," Valid data:");  }
  fprintf(fp," Ref=%s Epoch=%s ", 
	  pt_head->Ref_Epoch().Print_time(20).c_str(),
	  Epoch().Print_time(20).c_str());
  fprintf(fp, " iframes=%d Frame_Length=%d ", 
	  pt_head->Frame_Number, pt_head->Frame_Length*8);
  fprintf(fp," StnID:%c%c Thread=%d Cmplx:%d %2dbit/%2dch \n", 
	  pt_head->Station_ID[0], pt_head->Station_ID[1], 
	  pt_head->Thread_Id, pt_head->Complex_data_flag, 
	  pt_head->Qbit(), pt_head->Nch());
  
  //  fprintf(fp,"----------------------------\n");
  
}
//======= VDIF libraries ==== to   here ======
