/** @file  VLBI_Header.hpp defines VLBI_Header Class, 
 *  which is a interface class for general 
 *   VLBI data Header such as K5, Mark5B, and VDIF
 * @author M.Sekido
 * @Data 2009.4.4
 */


#ifndef __VLBI_Header_HPP__
#define __VLBI_Header_HPP__

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
using namespace std;

extern "C"{
#include "stdint.h"
}

#include "time_lib.hpp"


/// VLBI Header types 
enum VLBI_frame_t {UN_KNOWN,  VDIF_Type, 
		 K5VSSP_Type, K5VSSP32_Type, Mk5B_Type };

/// @class VLBI_Header class as abstract interface class for 
///  abstract general VLBI-Framing data
class VLBI_frame {
protected:
  int framesize; //< Frame (data part) size (byte)
  int headsize;  //< Header size (Bytes)
public:
  /// Constructor
  VLBI_frame(const int f, const int h):framesize(f),headsize(h){}
  virtual VLBI_frame_t Type()const=0;
  int Headsize()const {return headsize;}
  int Dframesize()const{return framesize;}
  /// Finding header part 
  /// @return  Number of bytes to find the header, or -1: for Error
};


//========== VDIF ===from Here  ====================
class VDIF_frame: public VLBI_frame{
  VDIF_Hdr vdif_hdr;   //< Header part 
  vector <uint8_t> dframe; //< Data frame array
public:
  VDIF_frame():VLBI_frame(0,32){}
  VLBI_frame_t Type()const{return VDIF_Type;}
  int Set_Framesize(const uint32_t sz){framesize =sz;dframe.resize(sz);}
}

//========== VDIF ===to   Here  ====================

//========== VSSP32 ===From here ====================


//========== VSSP32 ===to Here  ====================
  //-----------------

//========== Mk5B ===From here ====================


//========== Mk5B ===to Here  ====================


#endif

// class VLBI_Header {
// protected:
//   vector <uint8_t> head_buf;
// public:
//   /// Constructor
//   VLBI_Header(const uint16_t size):head_buf(size){}
//   /// Finding header part 
//   /// @return  Number of bytes to find the header, or -1: for Error
//   virtual int32_t Hd_find(const uint8_t*  buf, const uint32_t nbyte)=0;
//   /// Print Header part 
//   virtual void Print_hd(FILE *fp=stderr)=0;
//   virtual int Framesize() =0; ///< return framesize including header (bytes)
//   virtual int Headsize()const =0;  ///< return header size (bytes)
//   virtual uint32_t Epoch_imjd()=0;    ///< return sec of year (sec)  
//   virtual double Epoch_sod()=0;    ///< return sec of year (sec)  
//   virtual VLBI_Hdr_t Header_type() const=0;
//   virtual ~VLBI_Header(){;} // cerr << "Destruct in VLBI_header\n";}
// };
