
#include "VLBI_Proto.hpp"



VLBI_Proto::VLBI_Proto(const ApplProtocol_t apt, 
		       const TransProtocol_t tpt):
  TransInfo(apt,tpt),_sock(0),  sent_bytes_initial(0), tbyte_at1sec(0),
  sent_bytes_at(0),  rate_adj_frq(5),vlbi_frame(0)
{
  memset(&addr,0,sizeof(addr));  
  memset(&st_rate_adj,0,sizeof(timeval)); 
  Create_sock();
}

/** Tx_Ready() 
 * Assign port/address and make connection/open for destination
 */
bool VLBI_Proto::Tx_Ready(){

  if(vlbi_frame->Framesize() ==0 ){
    Logsystem::error_logger()->Tstamp() 
      << "E VLBI_Proto::Tx_Ready(): vlbi_frame Framesize == 0.\n";
    cerr << "E VLBI_Proto::Tx_Ready(): vlbi_frame Framesize == 0."
	 << endl;
    return false;
  }

  if(hostname.empty()){
    Logsystem::error_logger()->Tstamp() 
      << "E VLBI_Proto::Tx_Ready(): Hostname is not set\n.";
    cerr << "E VLBI_Proto::Tx_Ready(): Hostname is not set"
	 << endl;   
    return false;  }
  if( data_port == 0 ){
    Logsystem::error_logger()->Tstamp() 
      << "E VLBI_Proto::Tx_Ready(): Port number is not set\n";
    cerr << "E VLBI_Proto::Tx_Ready(): Port number is not set"
	 << endl;   
    return false;  }
  if( _sock == 0 ){
    Logsystem::error_logger()->Tstamp() 
      << "E VLBI_Proto::Tx_Ready(): Socket is not set\n";
    cerr << "E VLBI_Proto::Tx_Ready(): Socket is not set"
	 << endl;   
    return false;  }

  if( speed == 0 ){
    Logsystem::error_logger()->Tstamp() 
      << "E VLBI_Proto::Tx_Ready(): TX speed data is not ready\n" 
      << "Maybe vlbi_frame->DAS_Info is not set.\n" ;
    cerr << "E VLBI_Proto::Tx_Ready(): TX speed data is not ready\n" 
	 << "Maybe vlbi_frame->DAS_Info is not set." << endl;
    return false;
  }
  
  if( transLayerProtocol == TCP_IO || transLayerProtocol == UDP_IO ){
    struct hostent *hostinfo =  gethostbyname(hostname.c_str() );
    if( hostinfo == NULL){
      Logsystem::error_logger()->Tstamp() 
	<< "E VLIB_Proto::Read(): Could not get hostinfo.\n";
      cerr << "Error in VLIB_Proto::Read(): Could not get hostinfo."
	   << endl;
      return false;
    }
    
    addr.sin_port=htons((uint16_t)data_port);
    addr.sin_family=AF_INET;
    addr.sin_addr.s_addr = 
      *((unsigned long *)hostinfo->h_addr_list[0]);
    char ip_str[80];    
    inet_ntop(AF_INET, &addr.sin_addr, ip_str, sizeof(ip_str));
    fprintf(stderr,"Set_Host(%s): Destination IP=%s:%d mtu=%d\n", 
	    hostname.c_str(),ip_str, data_port, _sock->get_mtu());
  }
  if( transLayerProtocol == FILE_IO ){
    struct stat st;
    memset(&st,0,sizeof(st));
    stat(hostname.c_str(), &st);
    if( S_ISDIR(st.st_mode) ){
      fprintf(stderr, "Error in VLBI_Proto::Tx_Ready(): [%s] is directory\n",
	      hostname.c_str());
      Logsystem::error_logger()->Tstamp() 
	<< "E VLBI_Proto::Tx_Ready(): [" << hostname << "] is directory\n";
	return false;
    }
  }
  if( transLayerProtocol == TCP_IO || transLayerProtocol == FILE_IO){
    if(_sock->connect(hostname, data_port) <0){
      fprintf(stderr,
	      "Error in VLBI_Proto::Tx_Ready(): Failed to connect (%s:%d) with TCP/FILE_IO.\n",
	      hostname.c_str(), data_port);
      Logsystem::error_logger()->Tstamp() 
	<<  "E VLBI_Proto::Tx_Ready(): Failed to connect ("
	<< hostname << ":" << data_port <<") with TCP/FILE_IO.\n";
      return false;
    }else{
      fprintf(stderr,"Info: socket connection to %s:%d successed.\n", 
	      hostname.c_str(), data_port);
      if(vdif_conf.Check_Key("Owner") && vdif_conf.Check_Key("Group")){
	uid_t vdif_uid = atoi(vdif_conf.Value("Owner").c_str());      
	gid_t vdif_gid = atoi(vdif_conf.Value("Group").c_str());      
	if(chown(hostname.c_str(), vdif_uid, vdif_gid) <0){
	  char buf[1024];
	  sprintf(buf,"E VLBI_Proto::Tx_Ready(): Chown(%s) failed.\n", hostname.c_str());
	  Logsystem::error_logger()->Tstamp()  << buf;
	  cerr << buf;
	}
      }
      
    } // end else

  }
  
  rate_meas.Disp_Txon();  // Displaying TX-rate ON

  //  if( vlbi_frame->Type() == VLBI_Frame::VDIF_frm)
  if( applLayerProtocol == SUDP_Proto)
    if( ComputePktSize() <0){
      cerr << "Error: Failed ComputePktSize." << endl;
      return false;
    }
      
  //  packet.resize(packet_size);
//   int32_t outPageSize = vlbi_frame->DAS_Inf().Speed()*4; // byte/sec
//   preSendBuf.resize(outPageSize); 
  
  if( UseTransInfSpeed()) // case TransInfo::speed is used.
    rate_meas.Set_Target_rate((double)(3.2e-5*speed));
  else{
    speed = vlbi_frame->DAS_Inf().Speed();
    rate_meas.Set_Target_rate((double)(3.2e-5*speed));
  }

  Logsystem::info_logger()->Tstamp()   << " VLBI_Proto::Tx_Ready() end.\n";

  rate_meas.Set_Mode(Rate_Meas::TX);
  applProtocolState = TX_READY;

  return true;

  } // End of Tx_Ready()
  

  void VLBI_Proto::Tx_Close(){
  if(transLayerProtocol == FILE_IO){
    _sock->close();
  }

  applProtocolState = TransInfo::STOP;

}


void VLBI_Proto::Timer_Start(){  
  gettimeofday(&st0,NULL);  
  st_rate_adj = st0; 
  sent_bytes_initial = sent_bytes_at = _sock->get_bytes_sent();
  rate_meas.Set_socket(_sock);
  rate_meas.start();
}  

/** Rx_Read() Assing/binding  address(UDP), Listen(),  
 */


bool  VLBI_Proto::Rx_Ready(const int port_no){
 
  uint16_t port;
//   if(_sock ==0){
//     cerr << "E VLBI_Proto::Rx_Ready(): Socket is not yet set" << endl;
//     return false;
//   }
  Create_sock();

  if(port_no ==0){
    port = data_port;
  }else
    port = port_no;
  string ip;
  int ret = _sock->bind(ip, port); 
  if(ret < 0){
    cerr << "E VLBI_Proto::Rx_Ready(): Failed to get address from Socket:"
	 << strerror(errno) << endl;
    return false;
  }
  
  switch(transLayerProtocol){
  case UDP_IO:
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    addr.sin_addr.s_addr = INADDR_ANY ;
    break;
  case TCP_IO:
    if(_sock->listen() < 0){
      cerr << "E VLBI_Proto::Rx_Ready():Failed to listen():"
	   << strerror(errno) << endl;
      return false;
    }
    break;
  case FILE_IO:
  default:
    cerr << "E VLBI_Proto::Rx_Ready(): FILE_IO for Rx is not supported yet"
	 << endl;
    break;
  }

  //  if( vlbi_frame->Type() == VLBI_Frame::VDIF_frm)  
  if( applLayerProtocol == SUDP_Proto )
    if( ComputePktSize() <0){
      cerr << "E VLBI_Proto::Rx_Ready(): Failed ComputePktSize" << endl;
      return false;
    }
  
  rate_meas.Set_Mode(Rate_Meas::RX);
  applProtocolState = RX_READY;
  return true;
} // End of Rx_Ready()


void VLBI_Proto::Set_VlbiFrame(VLBI_Frame * vf){
  if(vf == 0){
    cerr << "Error @VLBI_Proto::Set_VlbiFrame(): vframe is 0."
	 << endl;
    return ;
  }
  vlbi_frame = vf;
  mtu = vf->Framesize();
  int32_t pktsize = mtu;
  switch(applLayerProtocol){
  case SUDP_Proto:
    //    mtu += SUDP_HEAD_SIZE;
    //    pktsize += SUDP_HEAD_SIZE;
    break;
  case RAW_Proto: 
    pktsize = vf->DAS_Inf().Speed()*4;
    pktsize += vf->Headsize();
    break;
  default:
    break;
  } 
  packet.resize(pktsize);
  speed = vlbi_frame->DAS_Inf().Bytes_sec()/4; // spped (words/sec)
}

/** Compute optimum packet size for VDIF within boudary of mtu size*/
int32_t  VLBI_Proto::ComputePktSize(const int32_t frsize){

  if(vlbi_frame ==0){
    cerr << "Error @VLBI_Proto::ComputePktSize(): vlbi_frame is empty" 
	 << endl;
    return -1;
  }

  int32_t Obytesec = vlbi_frame->DAS_Inf().Speed()/2; // (8byte/sec)
  int32_t framesize = frsize;
  if(transLayerProtocol == UDP_IO){ // In case of UDP
    int b_max = mtu - UDPIP_HD_SIZE;
    b_max -= ApplProtoHeadSize();
    if(vlbi_frame->Type() == VLBI_Frame::VDIF_frm) 
      b_max -= vlbi_frame->Headsize();
    int nmax = b_max/8; // Frame size must be multiple of 8byte(VDIF)
    int flag=0;
    int i=nmax;
    while(flag == 0 && i > 0){
      if( Obytesec % i == 0) flag =1;
      else i--;
    }
    int pld_size = i * 8;
    framesize = pld_size + vlbi_frame->Headsize();
    if( (Obytesec * 8) % pld_size !=0 ){
      cerr << "Error at Compute_pktsize: Error, pld_size=" 
	   << pld_size << endl;
      exit(1);
    }
  }else{ // In case of TCP/File
    if( frsize > 0 && (Obytesec * 8) % frsize != 0 ){
      cerr << "Warning at VLBI_Proto::ComputePktSize(): Speed("
	   << Obytesec*8 << "B/s) % frisize("<< frsize << ")=!0 "
	   << endl;
      int n = Obytesec*8/frsize;
      framesize = Obytesec*8/(n+1);
      cerr << " frsize was modified to "<< framesize << endl;
    }
    framesize += vlbi_frame->Headsize();
  }
  
  vlbi_frame->Set_Framesize(framesize);
  int32_t pktsize =  framesize + ApplProtoHeadSize();
  packet.resize(pktsize);
  return pktsize;
}


int VLBI_Proto::Set_DasInf(const DAS_Info& das){
  if(vlbi_frame ==0){
    cerr << "E VLBI_Proto::Set_DasInf(): vlbi_frame is empty."
	 << endl;
    return -1;
  }
  vlbi_frame->Set_DasInfo(das);

  if( ApplProtocol() == TransInfo::RAW_Proto){
    int32_t pktsize =  vlbi_frame->Framesize() + ApplProtoHeadSize();
    packet.resize(pktsize);
  }
  return 0;
}


int VLBI_Proto::Create_sock(){
    //  if(m > 0) mtu= m;
    //  packet.resize(mtu);
  
    switch(transLayerProtocol){
    case TCP_IO: 
	if(_sock!=0) delete _sock;
	_sock = new TCPSocket(mtu);
	_sock->set_so_sndbuf(SNDBUFSIZE);
	break;
    case UDP_IO:
	if(_sock!=0) delete _sock;
	//    if(_udpsock!=0) delete _udpsock;
	//    _udpsock = new UDPSocket(mtu);
	_sock = new UDPSocket(mtu);
	//    _sock = _udpsock;
	//    _udpsock->set_so_sndbuf(SNDBUFSIZE);
	_sock->set_so_sndbuf(SNDBUFSIZE);
	break;
    case FILE_IO:
	if(_sock!=0) delete _sock;
	_sock = new FileSocket(mtu);
	break;
    default:
	cerr << "Error in VIDF_Proto::Create_sock: "
	     << "Unsupported protocol at present.\n";
	return -1;
	break;
    }
    return 0;
}
